import React, { useEffect } from "react";
import {
	Box,
	Button,
	Checkbox,
	Dialog,
	DialogActions,
	DialogContent,
	DialogTitle,
	FormControl,
	FormControlLabel,
	FormHelperText,
	FormLabel,
	InputLabel,
	MenuItem,
	Select,
	TextField,
} from "@mui/material";
import CloseIcon from "@mui/icons-material/Close";
import { useForm, Controller } from "react-hook-form";

interface AddRoleModalProps {
	open: boolean;
	handleClose: () => void;
	handleCreate: (data: FormData) => void;
	handleUpdate: (
		id: string,
		name: string,
		resource: {
			moduleName: string;
			moduleId: string;
			permissions: string[];
		}[]
	) => void;
	editRow: any;
	data: {
		resource: {
			moduleName: string;
			moduleId: string;
			permissions: string[];
		}[];
	}; // Assuming `data` is passed as a prop
}

interface FormData {
	name: string;
	resource: {
		moduleName: string;
		moduleId: string;
		permissions: string[];
	}[];
}

const AddRoleModal: React.FC<AddRoleModalProps> = ({
	open,
	handleClose,
	handleCreate,
	handleUpdate,
	editRow,
	data,
}) => {
	const {
		control,
		handleSubmit,
		formState: { errors },
		setValue,
		reset,
		getValues, // Access getValues from the form methods here
	} = useForm<FormData>({
		defaultValues: {
			name: "",
			resource: [], // Ensure resource is initialized as an empty array
		},
	});


	useEffect(() => {
		if (editRow) {
			setValue("name", editRow.name);
			setValue("resource", editRow.resource);
		}
	}, [editRow, setValue]);

	// Handles permissions checkbox change for a specific resource
	const handlePermissionChange = (
		resourceIndex: number,
		permission: string,
		checked: boolean
	) => {
		const updatedResources = [...getValues().resource]; // Use getValues to get the current form values
		const resource = updatedResources[resourceIndex];

		if (checked) {
			// Add permission if checked
			resource.permissions = [
				...new Set([...resource.permissions, permission]),
			];
		} else {
			// Remove permission if unchecked
			resource.permissions = resource.permissions.filter(
				(p) => p !== permission
			);
		}

		setValue("resource", updatedResources); // Update the resource field in form state
	};

	const onSubmit = (data: FormData) => {
		if (editRow) {
			handleUpdate(editRow.id, data.name, data.resource);
		} else {
			handleCreate(data);
		}

		handleClose();
		reset();
	};

	return (
		<Dialog
			open={open}
			onClose={handleClose}
			maxWidth="md"
			fullWidth
			PaperProps={{
				component: "form",
				onSubmit: handleSubmit(onSubmit),
			}}
		>
			<DialogTitle
				sx={{
					display: "flex",
					alignItems: "center",
					justifyContent: "space-between",
				}}
			>
				{editRow ? "Edit Role" : "Add Role"}
				<Box
					onClick={handleClose}
					sx={{
						cursor: "pointer",
						display: "flex",
						alignItems: "center",
					}}
				>
					<CloseIcon />
				</Box>
			</DialogTitle>

			<DialogContent>
				{/* Role Name Field */}
				<Controller
					name="name"
					control={control}
					rules={{
						required: "Role Name is required",
						minLength: {
							value: 3,
							message: "Minimum 3 characters required",
						},
						maxLength: {
							value: 30,
							message: "Maximum 30 characters allowed",
						},
					}}
					render={({ field }) => (
						<TextField
							{...field}
							autoFocus
							required
							margin="dense"
							label="Role Name"
							type="text"
							fullWidth
							variant="standard"
							error={!!errors.name}
							helperText={errors.name?.message}
						/>
					)}
				/>

				{/* Resource Field */}
				<Controller
					name="resource"
					control={control}
					rules={{ required: "Resource is required" }}
					render={({ field }) => (
						<FormControl
							fullWidth
							margin="dense"
							error={!!errors.resource}
						>
							<InputLabel>Resource</InputLabel>
							<Select
								{...field}
								label="Resource"
								required
								fullWidth
								variant="standard"
							>
								{/* Mapping over the resource array to display the options */}
								{data.resource?.map((resourceItem, index) => (
									<MenuItem
										key={index}
										value={resourceItem.moduleId}
									>
										{resourceItem.moduleName}
									</MenuItem>
								))}
							</Select>
							<FormHelperText>
								{errors.resource?.message}
							</FormHelperText>
						</FormControl>
					)}
				/>

				{/* Permissions Checkbox Fields for each resource */}
				{getValues().resource &&
					getValues().resource.length > 0 &&
					getValues().resource.map((resource, resourceIndex) => (
						<React.Fragment key={resourceIndex}>
							<FormControl fullWidth margin="dense">
								<FormLabel>
									{resource.moduleName} Permissions
								</FormLabel>
								<FormControlLabel
									control={
										<Checkbox
											value="view"
											checked={resource.permissions.includes(
												"view"
											)}
											onChange={(e) =>
												handlePermissionChange(
													resourceIndex,
													"view",
													e.target.checked
												)
											}
										/>
									}
									label="View"
								/>
								<FormControlLabel
									control={
										<Checkbox
											value="edit"
											checked={resource.permissions.includes(
												"edit"
											)}
											onChange={(e) =>
												handlePermissionChange(
													resourceIndex,
													"edit",
													e.target.checked
												)
											}
										/>
									}
									label="Edit"
								/>
								<FormControlLabel
									control={
										<Checkbox
											value="delete"
											checked={resource.permissions.includes(
												"delete"
											)}
											onChange={(e) =>
												handlePermissionChange(
													resourceIndex,
													"delete",
													e.target.checked
												)
											}
										/>
									}
									label="Delete"
								/>
								<FormHelperText>
									{
										errors.resource?.[resourceIndex]
											?.permissions?.message
									}
								</FormHelperText>
							</FormControl>
						</React.Fragment>
					))}
			</DialogContent>

			<DialogActions>
				<Button onClick={handleClose}>Cancel</Button>
				<Button type="submit">{editRow ? "Update" : "Create"}</Button>
			</DialogActions>
		</Dialog>
	);
};

export default AddRoleModal;
